#!/usr/bin/env python
# coding: utf-8

import grab
import os
import csv
import sys
import argparse
import getting_time
from Class_ResultsSpider import ResultsSpider
from Class_ScheduleSpider import ScheduleSpider
from Class_RanksSpider import RanksSpider
from grab.spider import Spider, Task

parser = argparse.ArgumentParser()
parser.add_argument("-u", type=str, default='f', help="f - for finished results ,s - for schedule, r - for getting ranks")
parser.add_argument("-c", type=str, default='m', help="m - for male, f - for female")
parser.add_argument("-t", type=str, default='l', help="l - for live rating, o - for official")
parser.add_argument("-d", type=bool, default=0, help="1 - for adding date to filename, 0 - not adding")
parser.add_argument("-n", type=int, default=0, help="input integer number < 0, for getting results or schedule in n days from today")
args = parser.parse_args()

def main():
	if args.u == 'f':
		spider = ResultsSpider(thread_number=2, args=args)
		spider.run()
	elif args.u == 's':
		spider = ScheduleSpider(thread_number=2)
		spider.run()
	elif args.u == 'r':
		spider = RanksSpider(thread_number=2, args=args)
		spider.run()
	else:
		sys.stderr.write('Unvalid input of option -u\n')

main()



#!/usr/bin/env python
# coding: utf-8

import grab
import os
import csv
import sys
import argparse
from grab.spider import Spider, Task

class ScheduleSpider(Spider):
    initial_urls = ['http://www.tennislive.net/', ]
    def task_initial(self, grab, task):
        '''
        '''
        date = ''
        if os.getenv('DATE') != None:
            date = os.getenv('DATE')
        self.base_url = self.initial_urls[0]
        task.url = 'http://www.tennislive.net/{}'.format(date)
        yield Task('schedule', url=task.url)

    def task_schedule(self, grab, task):
        '''
        Go to the site with schedule
        '''
        xpath = '//div[@class = "full"]/ul/li/a'
        for elem in grab.doc.select(xpath):
            if elem.text() == "Scheduled":
                new_url = elem.attr('href')
                sys.stdout.write("Schedule:")
                yield Task('get_pairs', new_url)

    def task_get_pairs(self, grab, task):
        '''
        Iterate on all mathes in schedule.
        '''
        xpath = '//tr[@class="pair" or @class="unpair"]/td[@class!="beg"] | tr[@class="pair" or @class="unpair"]/td[@class="detail"]/div[@class="head2head"]'
        for elem in grab.doc.select(xpath):
            if elem.attr('class') == "head2head":
                el = elem.select('a')
                stats_url = el.attr('href')
                yield Task('get_info', stats_url)

    def task_get_info(self, grab, task):
        '''
        Get info about match in schedule.
        '''
        NUM_OF_COLUMN = 6
        xpath = '//div[@class="player_matches"]/table/tr/td'
        i = 0
        match = []
        for elem in grab.doc.select(xpath):
            if i < NUM_OF_COLUMN:
                if elem.text() != '':
                    match.append(str(elem.text()))
                i += 1
            elif i == NUM_OF_COLUMN:
                if elem.text() != '':
                    match.append(str(elem.text()))
                break
        flag = False
        for elem in match:
            if elem != 'no matches found':
                sys.stdout.write(elem + ', ')
                flag = True
        if flag:
            sys.stdout.write('\n')


def parse_score(string):
    '''
    '''
    score = string.split(',')
    res = []
    for games in score:
        if len(games.strip()) > 1:
            games = games.strip()
            e = games.split('-')
            if abs(int(e[0])-int(e[1])) != 2 and int(e[0]) > 59 or int(e[1]) > 59:
                if e[0][0] == '7' and e[1][0] == '6':
                    e[0] = '7'
                    e[1] = '6'
                elif e[0][0] == '6' and e[1][0] == '7':
                    e[0] = '6'
                    e[1] = '7'
            res.append(int(e[0]))
            res.append(int(e[1]))
    return res

def retired_score(string):
    '''
    '''
    a = string.split('retired')
    score = a[0]
    res = parse_score(score)
    name = a[1].strip()
    return (res, name)

def who_wins(res):
    '''
    '''
    num_of_set = len(res) // 2
    count_p1 = 0
    count_p2 = 0
    for i in range(num_of_set):
        if res[2 * i] > res[2 * i + 1]:
            count_p1 += 1
        else:
            count_p2 += 1
    return count_p1 > count_p2



# coding: utf-8

import time
import grab
import os
import csv
import sys
import argparse
import getting_time
from parse_score import parse_score, retired_score, who_wins
from getting_time import get_time, get_date, make_date
from grab.spider import Spider, Task

class ResultsSpider(Spider):
    initial_urls = ['http://www.tennislive.net/',]
    def task_initial(self, grab, task):
        '''
        '''
        date = ''
        self.base_url = self.initial_urls[0]
        n = self.args.n
        flag = False
        OK = True
        if n:
            if n > 0:
                sys.stderr.write("Error: option -n must be less then 0\n")
                OK = False
            else:
                t = get_time()
                date = get_date(t[0], t[1], t[2], n)
            flag = True
        if os.getenv('TENNIS_DATE'):
            if flag:
                sys.stderr.write("Error: argument TENNIS_DATE and option -n can't be inputed together\n")
                OK = False
            else:
                date = os.getenv('TENNIS_DATE')
                OK = True
        if OK:
            atp_url = ('http://www.tennislive.net/atp-men/{}'.format(date))
            wta_url = ('http://www.tennislive.net/wta-women/{}'.format(date))
            yield Task('atp_tournament_list', url=atp_url, date=date)
            yield Task('wta_tournament_list', url=wta_url, date=date)


    def task_atp_tournament_list(self, grab, task):
        '''
        Get the list of atp tournaments.
        '''
        civility = "male"
        xpath = "//div[@class='tour_box']/ul/li[@class='menu_main']/a"
        for tournament in grab.doc.select(xpath):
            tournament_url = tournament.attr('href')
            tournament_name = tournament.attr('title')
            if tournament_name not in ["ATP ranking", "WTA ranking", "ALL TOURNAMENTS"]:
                yield Task('tournament_info', tournament_url, tournament_name=tournament_name, civility=civility, date=task.date)   
    
    def task_wta_tournament_list(self, grab, task):
        '''
        Get the list of wta tournaments.
        '''
        civility = "female"
        xpath = "//div[@class='tour_box']/ul/li[@class='menu_main']/a"
        for tournament in grab.doc.select(xpath):
            tournament_url = tournament.attr('href')
            tournament_name = tournament.attr('title')
            if tournament_name not in ["ATP ranking", "WTA ranking", "ALL TOURNAMENTS"]:
                yield Task('tournament_info', tournament_url, tournament_name=tournament_name, civility=civility, date=task.date)   

    def task_tournament_info(self, grab, task):
        '''
        Get the site with all finished mathes in tournament
        '''
        NAME_OF_TOURNAMENT = '-'
        begins_date = '-'
        xpath = '//title'
        for elem in grab.doc.select(xpath):
            try:
                title = elem.text().split('/')
                NAME_OF_TOURNAMENT = title[0]
                if len(title[1].split('-')) == 3:
                    begins_date = title[1].split('-')[1]
            except IndexError:
                sys.stderr.write("Not enogh info in title\n")
                print(elem.text())

        xpath ='//ul[@id = "topmenu_full"]/li/a'
        for elem in grab.doc.select(xpath):
            if elem.text() == 'Finished':
                fin_url = elem.attr('href')
                yield Task('get_pairs', fin_url, civility=task.civility, NAME_OF_TOURNAMENT=NAME_OF_TOURNAMENT, begins_date=begins_date, date=task.date)
        
    def task_get_pairs(self, grab, task):
        '''
        Iterate on the list of finished mathes in tournament
        '''
        xpath = '//tr[@class="pair" or @class="unpair"]/td[@class!="beg"] | tr[@class="pair" or @class="unpair"]/td[@class="detail"]/div[@class="head2head"]'
        for elem in grab.doc.select(xpath):
            if elem.attr('class') == "head2head":
                el = elem.select('a')
                stats_url = el.attr('href')
                yield Task('get_stats', stats_url, civility=task.civility, NAME_OF_TOURNAMENT=task.NAME_OF_TOURNAMENT, begins_date=task.begins_date, date=task.date)



    def task_get_stats(self, grab, task):
        '''
        Get all infromation about the finished matches, including stats and info about players.
        '''
        NUM_OF_COLUMN = 6
        NUM_OF_COLUMN_IN_STATS_TABLE = 3
        COLUMS_FOR_MATCH_STATS = 16
        COLUMN_AFTER_MATCH_INFO = 18
        COLUMS_FOR_PLAYERS_INFO = 36
        NAME_PLAYER1 = 5
        NAME_PLAYER2 = 6
        COLUMS_IN_PLAYER_INFO = 9
        COLUMS_AFTER_STATS = 34
        COLUMS_AFTER_STATS_AND_PLAYER1 = 43
        COLUMS_AFTER_STATS_AND_PAIR1 = 52
        COLUMS_AFTER_STATS_AND_PAIR1_AND_PLAYER3 = 61
        NUM_BEFORE_SCORE = 9
        RES_1 = 7
        RES_2 = 8

        row = []
        row.append(task.NAME_OF_TOURNAMENT)
        row.append(task.begins_date)
        
        xpath = '//div[@class="player_matches"]/table/tr/td' #данные о матче - дата, раунд, имена, счет
        
        i = 0
        for elem in grab.doc.select(xpath):
            if i < NUM_OF_COLUMN and i not in [2, 4]:
                row.append(str(elem.text()))
                i += 1
            elif i == 2:
                row.append(str(task.civility))
                row.append(str(elem.text()))
                i += 1
            elif i == 4:
                for k in range(12):
                    row.append('-')
                string = str(elem.text()) 
                if 'retired' not in string and 'walk over' not in string:
                    res = parse_score(string)
                    row[NUM_BEFORE_SCORE : (NUM_BEFORE_SCORE + len(res))] = res[:]
                    winner = who_wins(res)
                    if winner:
                        row[RES_1] = 'W'
                        row[RES_2] = 'L'
                    else:
                        row[RES_2] = 'W'
                        row[RES_1] = 'L'
                elif 'walk over' in string:
                    name = string.split('-')[0].strip()
                    if name == row[NAME_PLAYER1]:
                        row[RES_1] = 'walk over'
                        row[RES_2] = 'W'
                    elif name == row[NAME_PLAYER2]:
                        row[RES_1] = 'W'
                        row[RES_2] = 'walk over'
                elif 'retired' in string:
                    res, name = retired_score(string)
                    row[NUM_BEFORE_SCORE : (NUM_BEFORE_SCORE + len(res))] = res[:]
                    if name == row[NAME_PLAYER1]:
                        row[RES_1] = 'retired'
                        row[RES_2] = 'W'
                    elif name == row[NAME_PLAYER2]:
                        row[RES_1] = 'W'
                        row[RES_2] = 'retired'
                i += 1
            elif i == NUM_OF_COLUMN:
                row.append(str(elem.text()))
                break

        xpath = '//table[@class="table_stats_match"]/tr/td'

        dic = {'1st SERVE %' : 1,
                '1st SERVE POINTS WON' : 2,
                '2nd SERVE POINTS WON' : 3,
                'BREAK POINTS WON' : 4,
                'TOTAL RETURN POINTS WON' : 5,
                'TOTAL POINTS WON' : 6,
                'DOUBLE FAULTS' : 7,
                'ACES' : 8,
                }

        for j in range(COLUMS_FOR_MATCH_STATS):
            row.append("-")

        i = 0
        stat = ''
        flag = True
        for elem in grab.doc.select(xpath):
            if flag:
                if i % NUM_OF_COLUMN_IN_STATS_TABLE == 2:
                    i += 1
                    flag = False
                else:
                    i += 1
            else:
                if i % NUM_OF_COLUMN_IN_STATS_TABLE != 0:
                    row[dic[stat] * 2 - (i % NUM_OF_COLUMN_IN_STATS_TABLE)%2 + COLUMN_AFTER_MATCH_INFO] = str(elem.text())
                    i += 1
                else:
                    stat = elem.text()
                    i += 1

        for j in range(COLUMS_FOR_PLAYERS_INFO):
            row.append("-")

        dic = {'Country' : 1,
                'Birthdate' : 2,
                'Height' : 3,
                'Weight' : 4,
                'Profi since' : 5,
                'Play' : 6,
                "Ranking's position" : 7,
                'Points' : 8,
                'Prize money' : 9,
                }

        if "/" not in row[NAME_PLAYER1]:
            last = 0
            position = ''
            xpath = '//div[@class="player_comp_info_left"]/a'
            for elem in grab.doc.select(xpath):
                position = elem.text()
            xpath = '//div[@class="player_comp_info_left"]/text() | //div[@class="player_comp_info_left"]/a'
            for elem in grab.doc.select(xpath):
                if not last:
                    row[COLUMS_AFTER_STATS + dic['Country']] = elem.text()
                    last = dic['Country']
                elif 'years' in elem.text():
                    row[COLUMS_AFTER_STATS + dic['Birthdate']] = elem.text()
                    last = dic['Birthdate']
                elif 'cm' in elem.text():
                    row[COLUMS_AFTER_STATS + dic['Height']] = elem.text()
                    last = dic['Height']
                elif 'kg' in elem.text():
                    row[COLUMS_AFTER_STATS + dic['Weight']] = elem.text()
                    last = dic['Weight']
                elif 'handed' in elem.text():
                    row[COLUMS_AFTER_STATS + dic['Play']] = elem.text()
                    last = dic['Play']
                elif '$' in elem.text():
                    row[COLUMS_AFTER_STATS + dic['Prize money']] = elem.text()
                    last = dic['Prize money']
                else:
                    try:
                        int(elem.text())
                        if elem.text() != position and last != 7:
                            row[COLUMS_AFTER_STATS + dic['Profi since']] = elem.text()
                            last = dic['Profi since']
                        elif elem.text() == position:
                            row[COLUMS_AFTER_STATS + dic["Ranking's position"]] = elem.text()
                            last = dic["Ranking's position"]
                        elif last == 7:
                            row[COLUMS_AFTER_STATS + dic['Points']] = elem.text()
                            last = dic['Points']
                    except ValueError:
                        pass

            last = 0
            position = ''
            xpath = '//div[@class="player_comp_info_right"]/a'
            for elem in grab.doc.select(xpath):
                position = elem.text()
            xpath = '//div[@class="player_comp_info_right"]/text() | //div[@class="player_comp_info_right"]/a'
            for elem in grab.doc.select(xpath):
                if not last:
                    row[COLUMS_AFTER_STATS_AND_PLAYER1 + dic['Country']] = elem.text()
                    last = dic['Country']
                elif 'years' in elem.text():
                    row[COLUMS_AFTER_STATS_AND_PLAYER1 + dic['Birthdate']] = elem.text()
                    last = dic['Birthdate']
                elif 'cm' in elem.text():
                    row[COLUMS_AFTER_STATS_AND_PLAYER1 + dic['Height']] = elem.text()
                    last = dic['Height']
                elif 'kg' in elem.text():
                    row[COLUMS_AFTER_STATS_AND_PLAYER1 + dic['Weight']] = elem.text()
                    last = dic['Weight']
                elif 'handed' in elem.text():
                    row[COLUMS_AFTER_STATS_AND_PLAYER1 + dic['Play']] = elem.text()
                    last = dic['Play']
                elif '$' in elem.text():
                    row[COLUMS_AFTER_STATS_AND_PLAYER1 + dic['Prize money']] = elem.text()
                    last = dic['Prize money']
                else:
                    try:
                        int(elem.text())
                        if elem.text() != position and last != 7:
                            row[COLUMS_AFTER_STATS_AND_PLAYER1 + dic['Profi since']] = elem.text()
                            last = dic['Profi since']
                        elif elem.text() == position:
                            row[COLUMS_AFTER_STATS_AND_PLAYER1 + dic["Ranking's position"]] = elem.text()
                            last = dic["Ranking's position"]
                        elif last == 7:
                            row[COLUMS_AFTER_STATS_AND_PLAYER1 + dic['Points']] = elem.text()
                            last = dic['Points']
                    except ValueError:
                        pass
        else:
            xpath = '//div[@class="player_comp_info_left"]/text()'
            num = 0
            flag = True
            for elem in grab.doc.select(xpath):
                if flag:
                    row[COLUMS_AFTER_STATS + dic['Country']] = elem.text()
                    flag = False
                else:
                    if 'years' in elem.text():
                        row[COLUMS_AFTER_STATS + COLUMS_IN_PLAYER_INFO * num + dic['Birthdate']] = elem.text()
                    elif 'cm' in elem.text():
                        row[COLUMS_AFTER_STATS + COLUMS_IN_PLAYER_INFO * num + dic['Height']] = elem.text()
                    elif 'kg' in elem.text():
                        row[COLUMS_AFTER_STATS + COLUMS_IN_PLAYER_INFO * num + dic['Weight']] = elem.text()
                    elif 'handed' in elem.text():
                        row[COLUMS_AFTER_STATS + COLUMS_IN_PLAYER_INFO * num + dic['Play']] = elem.text()
                    else:
                        if elem.text().isnumeric():
                            row[COLUMS_AFTER_STATS + COLUMS_IN_PLAYER_INFO * num + dic['Profi since']] = elem.text()                                            
                        elif elem.text() != '':
                            row[COLUMS_AFTER_STATS + dic['Country']] = elem.text()
                            num += 1

            xpath = '//div[@class="player_comp_info_right"]/text()'
            num = 0
            flag = True
            for elem in grab.doc.select(xpath):
                if flag:
                    row[COLUMS_AFTER_STATS_AND_PAIR1 + dic['Country']] = elem.text()
                    flag = False
                else:
                    if 'years' in elem.text():
                        row[COLUMS_AFTER_STATS_AND_PAIR1 + COLUMS_IN_PLAYER_INFO * num + dic['Birthdate']] = elem.text()
                    elif 'cm' in elem.text():
                        row[COLUMS_AFTER_STATS_AND_PAIR1 + COLUMS_IN_PLAYER_INFO * num + dic['Height']] = elem.text()
                    elif 'kg' in elem.text():
                        row[COLUMS_AFTER_STATS_AND_PAIR1 + COLUMS_IN_PLAYER_INFO * num + dic['Weight']] = elem.text()
                    elif 'handed' in elem.text():
                        row[COLUMS_AFTER_STATS_AND_PAIR1 + COLUMS_IN_PLAYER_INFO * num + dic['Play']] = elem.text()
                    else:
                        if elem.text().isnumeric():
                            row[COLUMS_AFTER_STATS_AND_PAIR1 + COLUMS_IN_PLAYER_INFO * num + dic['Profi since']] = elem.text()                                            
                        elif elem.text() != '':
                            row[COLUMS_AFTER_STATS_AND_PAIR1_AND_PLAYER3 + dic['Country']] = elem.text()
                            num += 1
        
        filename = os.getenv('TENNIS_FILENAME')
        n = self.args.n
        d = self.args.d
        if d:
            if not task.date:
                t = get_time()
                task.date = make_date(t[0], t[1], t[2])
            ndate = task.date.split('-')
            split_name = filename.split('.')
            f = '{}_{}_{}_{}.{}'.format(split_name[0], ndate[0], ndate[1], ndate[2], split_name[1])
        else:
            f = '{}'.format(filename)
        with open(f, 'a') as res:
            writer = csv.writer(res)
            writer.writerow(row)



# coding: utf-8

import grab
import os
import getting_time
from getting_time import get_time, make_date
import csv
import sys
import argparse
from grab.spider import Spider, Task

class RanksSpider(Spider):
    initial_urls = ['http://live-tennis.eu/']
    def task_initial(self, grab, task):
        '''
        '''
        self.base_url = self.initial_urls[0]
        if self.args.c == 'm':
            if self.args.t == 'l':
                yield Task('get_ranks', url='http://live-tennis.eu/')
            elif self.args.t == 'o':
                yield Task('get_ranks', url='http://live-tennis.eu/official_atp_ranking')
            else:
                sys.stderr.write('Unvalid input of utility -t\n')
        elif self.args.c == 'f':
            if self.args.t == 'l':
                yield Task('get_ranks', url='http://live-tennis.eu/wta-live-ranking')
            elif self.args.t == 'o':
                yield Task('get_ranks', url='http://live-tennis.eu/official-wta-ranking')
            else:
                sys.stderr.write('Unvalid input of utility -t\n')
        else:
            sys.stderr.write('Unvalid input of option -c\n')

    def task_get_ranks(self, grab, task):
        '''
        Get info about ranks.
        '''
        NUM_OF_STRINGS = 5500
        NUM_OF_COLUMN = 11

        t = get_time()
        date = make_date(t[0], t[1], t[2])

        filename = os.getenv('TENNIS_FILENAME')
        xpath = '//tr/td'
        row = []
        flag = False
        with open('%s' %filename, 'a') as res:
            writer = csv.writer(res)
            for elem in grab.doc.select(xpath):
                if not flag:
                    if elem.text() == '1':
                        j = 1
                        row.append(elem.text())
                        flag = True
                else:
                    if j < NUM_OF_STRINGS:
                        if j % NUM_OF_COLUMN == (NUM_OF_COLUMN-1):
                            row.append(elem.text())
                            row.append(date)
                            writer.writerow(row)
                            row = []
                            j += 1

                        else:
                            row.append(elem.text())
                            j += 1
                    else:
                        break



# coding: utf-8

#Настроить внутренние функции, чтобы можно было выбирать, какуб информацию об игроке получать.

import grab
import os
import csv
import sys
import argparse
from grab.spider import Spider, Task

class H2HSpider:
    pass


from setuptools import setup, find_packages
from os.path import join, dirname

setup(
    name = "TennisSpider",
    version = "0.2",
    packages = ['TennisSpider'],
    author = "Dmitry Dubov",
    author_email = "dmitry.s.dubov@gmail.com",
    description = "Package for parsing info about tennis results and stats.",
    long_description=open(join(dirname(__file__), 'README.md')).read(),
    license = "PSF",
    keywords = "Tennis, Results",
    url = "https://github.com/cs-hse-projects/DataSpider_Dubov", 
    entry_points={
        'console_scripts':
            ['get_tennis=TennisSpider.main:main']
        }
)


import time

def get_time():
    '''
    Returns tuple with the date in format (year, month, day)
    '''
    t = time.localtime(time.time())
    year = t.tm_year
    month = t.tm_mon
    day = t.tm_mday
    return ((year, month, day))

def make_date(year, month, day):
    '''
    Make from three parametres year, month, day, date_string in format: "Year-Month_Day". Also, if day or month < 10, added 0 before it.
    '''
    new_year = str(year)
    if  day < 10:
        new_day = '0' + str(day)
    else:
        new_day = str(day)
    if month < 10:
        new_month = '0' + str(month)
    else:
        new_month = str(month)
    string = new_year + '-' + new_month + '-' + new_day  
    return string

#Getting new date
def is_leap(year):
    '''
    Check, that year is leap or not.
    '''
    if ((year % 4 == 0) and (year % 100 != 0)) or (year % 400 == 0):
        return True
    else:
        return False

def koldays(m, y):
    '''
    Returns the number of days in month.
    '''
    month_31 = [1, 3, 5, 7, 8, 10, 12]
    month_30 = [4, 6, 9, 11]
    if m in month_31:
        return 31;
    elif m in month_30:
        return 30;
    elif m == 2:
        if is_leap(y):
            return 29
        else:
            return 28

def nextday(d, m, y, backi):
    '''
    Returns the next or previous day, depends on argument backi.
    '''
    if (1 <= d + 1*backi) and (d + 1*backi <= koldays(m, y)):
        d = d + 1*backi
    else:
        if (1 <= m + 1*backi) and (m + 1*backi <= 12):
            m = m + 1*backi;
            d = 1 if backi == 1 else koldays(m, y)
        else:
            d = 1 if backi == 1 else koldays(12, y)
            m = 1 if backi == 1 else 12
            y = y + 1*backi
    return d, m, y

def get_date(year, month, day, n):
    '''
    Returns the new date.
    '''
    backi = 1 if n > 0 else -1
    i = 0
    while abs(i) < abs(n):
        (day, month, year) = nextday(day, month, year, backi)
        i += 1*backi
    return make_date(year, month, day)
